#!/usr/bin/env node
'use strict';

var cli = require('@effect/cli');
var NodeContext = require('@effect/platform-node/NodeContext');
var NodeRuntime = require('@effect/platform-node/NodeRuntime');
var effect = require('effect');
var Effect3 = require('effect/Effect');
var zod = require('zod');
var platform = require('@effect/platform');
var mcp_js = require('@modelcontextprotocol/sdk/server/mcp.js');
var stdio_js = require('@modelcontextprotocol/sdk/server/stdio.js');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var NodeContext__namespace = /*#__PURE__*/_interopNamespace(NodeContext);
var NodeRuntime__namespace = /*#__PURE__*/_interopNamespace(NodeRuntime);
var Effect3__namespace = /*#__PURE__*/_interopNamespace(Effect3);

var layerLogger = effect.Logger.replace(
  effect.Logger.defaultLogger,
  effect.Logger.prettyLogger({
    // We are currently using stdio as MCP transport, therefore we cannot log
    // on stdout, we have to log in stderr.
    stderr: true,
    colors: true,
    mode: "tty"
  })
);
var ApiKey = effect.Schema.String.pipe(
  effect.Schema.pattern(/^rember_[a-f0-9]{32}$/),
  effect.Schema.brand("ApiKey")
);
var Note = effect.Schema.Struct({
  text: effect.Schema.String.pipe(effect.Schema.maxLength(2e3))
});
var Notes = effect.Schema.Array(Note).pipe(effect.Schema.maxItems(50));
var ErrorApiKeyInvalid = class extends effect.Schema.TaggedError()(
  "Api/ApiKeyInvalid",
  { message: effect.Schema.String }
) {
};
var ErrorReachedLimitRateLimiter = class extends effect.Schema.TaggedError()(
  "Api/ReachedLimitRateLimiter",
  { message: effect.Schema.String }
) {
};
var ErrorReachedLimitUsageTracker = class extends effect.Schema.TaggedError()(
  "Api/ReachedLimitUsageTracker",
  { message: effect.Schema.String }
) {
};
var endpointGenerateCardsAndCreateRembs = platform.HttpApiEndpoint.post(
  "generateCardsAndCreateRembs",
  "/v1/generate-cards-and-create-rembs"
).setPayload(
  effect.Schema.Struct({
    version: effect.Schema.Literal("1"),
    notes: Notes
  })
).setHeaders(
  effect.Schema.Struct({
    "x-api-key": ApiKey,
    "x-source": effect.Schema.String
  })
).addError(ErrorApiKeyInvalid, { status: 401 }).addError(ErrorReachedLimitUsageTracker, { status: 403 }).addError(ErrorReachedLimitRateLimiter, { status: 429 });
var groupV1 = platform.HttpApiGroup.make("v1").add(endpointGenerateCardsAndCreateRembs);
var apiRember = platform.HttpApi.make("Rember").add(groupV1).prefix("/api");
var Rember = class _Rember extends effect.Context.Tag("Rember")() {
  static layer() {
    return effect.pipe(
      effect.Layer.effect(_Rember, makeRember),
      effect.Layer.provide(platform.FetchHttpClient.layer)
    );
  }
};
var makeRember = effect.Effect.gen(function* () {
  const client = yield* platform.HttpApiClient.make(apiRember, { baseUrl: "https://www.rember.com/" });
  const apiKeyEnc = yield* effect.Config.string("REMBER_API_KEY");
  const apiKey2 = yield* effect.pipe(
    apiKeyEnc,
    effect.Schema.decodeUnknown(ApiKey),
    effect.Effect.catchTag("ParseError", () => effect.Effect.dieMessage("Invalid API key"))
  );
  const generateCardsAndCreateRembs = ({ notes }) => client.v1.generateCardsAndCreateRembs({
    payload: { version: "1", notes },
    headers: { "x-api-key": apiKey2, "x-source": "rember-mcp" }
  });
  return {
    generateCardsAndCreateRembs
  };
});
var ServerMCP = class _ServerMCP extends effect.Context.Tag("ServerMCP")() {
  static layer({ name, tools, version }) {
    return effect.Layer.scoped(_ServerMCP, makeServerMCP({ name, version, tools }));
  }
};
var makeServerMCP = ({ name, tools, version }) => effect.Effect.gen(function* () {
  const server = yield* effect.Effect.sync(() => new mcp_js.McpServer({ name, version }));
  const transport = yield* effect.Effect.sync(() => new stdio_js.StdioServerTransport());
  const runtime = yield* effect.Effect.runtime();
  for (const [name2, tool] of effect.Record.toEntries(tools)) {
    yield* effect.Effect.sync(
      () => server.tool(
        name2,
        tool.description,
        tool.schemaInput,
        (args, { signal }) => effect.pipe(
          tool.effect(args),
          // Report errors
          effect.Effect.tapErrorCause((cause) => {
            if (effect.Cause.isInterruptedOnly(cause)) {
              return effect.Effect.void;
            }
            return effect.Effect.logError(cause);
          }),
          (effect$1) => effect.Runtime.runPromise(runtime)(effect$1, { signal })
        )
      )
    );
  }
  yield* effect.Effect.acquireRelease(
    // `server.connect` starts the transport and starts listening for messages
    effect.Effect.promise(() => server.connect(transport)),
    // `server.close` closes the transport
    () => effect.Effect.promise(() => server.close())
  );
  return {
    server
  };
});

// src/bin.ts
var apiKey = effect.pipe(
  cli.Options.text("api-key"),
  cli.Options.withSchema(ApiKey),
  cli.Options.map(effect.Redacted.make),
  cli.Options.optional
);
var command = cli.Command.make("rember-mcp", { apiKey }, ({ apiKey: apiKey2 }) => Effect3__namespace.gen(function* () {
  const rember = yield* Rember;
  const layerServerMCP = ServerMCP.layer({
    name: "rember",
    version: "1.0.7",
    tools: {
      "create_flashcards": {
        description: `
A tool to generate spaced-repetition flashcards in Rember.

What is Rember?
Rember is a modern spaced-repetition system based on *rembs*. A remb is a concise note focused on a single concept or idea you want to remember along with a few flashcards testing that concept or idea. In Rember you can create rembs and review their flashcards, just like in Anki or other traditional spaced-repetition systems. Rember also allows exporting rembs to Anki.

When to use this tool?
Use this tool when the user wants to remember one or more ideas, or the user explicitly asks to add something to Rember. This tools allows users to review over time and internalize new concepts or ideas learned from their conversation with you. Since you can work with pdfs and, given the right tool, with webpages and other resources, you can help users create flashcard about pretty much any kind of resource.

Input:
A list of notes which will be sent to the Rember API. Rember will turn each note into a remb, by generating flashcards using AI, independently from this conversation with you. Rember will often create 4-5 flashcards for each single note. Notes are the natural organizational unit for spaced-repetition flashcards, they allow users to quickly search, organize and interact with flashcards.

Output:
The tool simply signals whether the operation succeded or failed.

Examples:
- After asking you a question or chatting with you, the user asks "help me remember this" or "create a few flashcards", you call this tool with one or two notes about the core ideas the user cares about
- The user asks you to create flashcards for a PDF or webpage, you extract the main points and send a note for each of them to this tool
- The user might follow up, and ask you to create more flashcards about something specific, you create one or two notes about that

Here are 9 rules for writing notes to send to the Rember API.

Rules:
1. Prefer summarizing information into a single note
2. Create many notes only if the user is trying to remember many different concepts or ideas
3. Focus only on essential concepts or ideas, unless the user explicitly asks about details
4. Notes should be atomic and mostly focused on a single concept or idea
5. Notes should be self-contained and make sense independently of other notes
6. Notes should be concise, get to the point and avoid unnecessary details or verbiage
7. Avoid repeating the same information across multiple notes
8. Use specific attributions when referencing sources (e.g., "Researcher Name states..." not "The article suggests...")
9. If the user asks something like "create N flashcards", explain: "I'll help you create notes on the key concepts you want to remember. Rember will automatically generate appropriate flashcards from each note. Would you like me to create notes on this topic instead?".
          `,
        schemaInput: {
          notes: zod.z.array(
            zod.z.object({
              text: zod.z.string().max(2e3).describe("The text content of the note")
            }).describe("A little note about a concept or idea")
          ).max(50).describe("A list of little notes"),
          source: zod.z.string().max(100).optional().describe(
            "The resource (e.g. article, book, pdf, webpage) the notes are about (e.g. 'Author - Title'). Omit this field unless the notes are about a specific concrete resource."
          )
        },
        // TODO: Fix inference of `input` type
        effect: (input) => effect.pipe(
          Effect3__namespace.gen(function* () {
            const _input = input;
            const notes = yield* effect.pipe(
              _input.notes,
              effect.Array.map(({ text }) => ({
                text: _input.source == void 0 ? text : `${text}

${_input.source}`
              })),
              effect.Schema.decodeUnknown(Notes)
            );
            yield* rember.generateCardsAndCreateRembs({ notes });
            return {
              content: [{ type: "text", text: `Generated cards for ${notes.length} rembs.` }]
            };
          }),
          // Handle usage limit reached error
          Effect3__namespace.catchTag("Api/ReachedLimitUsageTracker", (_) => Effect3__namespace.succeed({
            content: [{
              type: "text",
              text: "You've reached your monthly limit. Visit https://www.rember.com/settings/account to upgrade to Pro and get more rembs."
            }],
            isError: true
          })),
          // Handle input error
          Effect3__namespace.catchTag("ParseError", (_) => Effect3__namespace.succeed({
            content: [{ type: "text", text: `Invalid input: ${_.message}` }],
            isError: true
          })),
          // Handle other errors
          Effect3__namespace.retry({
            times: 3,
            schedule: effect.Schedule.exponential("2 second")
          }),
          Effect3__namespace.timeout("30 seconds"),
          Effect3__namespace.catchAll(
            (_) => Effect3__namespace.succeed({
              content: [{ type: "text", text: _.message }],
              isError: true
            })
          )
        )
      }
    }
  });
  yield* effect.Layer.launch(layerServerMCP);
}).pipe(
  Effect3__namespace.provide(Rember.layer()),
  Effect3__namespace.provide(
    effect.pipe(
      effect.ConfigProvider.fromJson(effect.Option.isSome(apiKey2) ? { REMBER_API_KEY: apiKey2 } : {}),
      effect.ConfigProvider.orElse(() => effect.ConfigProvider.fromEnv()),
      (_) => effect.Layer.setConfigProvider(_)
    )
  )
));
var run = cli.Command.run(command, {
  name: "Rember MCP server",
  version: "1.0.7"
});
run(process.argv).pipe(
  // Report errors, this needs to happen:
  // - After the creation of our main layers, to report errors in the layer construction
  // - Before providing `layerLogger` so that the errors are reported with the correct
  //   logger
  // Note that we set `disableErrorReporting: true` in `NodeRuntime.runMain`.
  Effect3__namespace.tapErrorCause((cause) => {
    if (effect.Cause.isInterruptedOnly(cause)) {
      return Effect3__namespace.void;
    }
    return Effect3__namespace.logError(cause);
  }),
  Effect3__namespace.provide(layerLogger),
  Effect3__namespace.provide(NodeContext__namespace.layer),
  NodeRuntime__namespace.runMain({ disableErrorReporting: true, disablePrettyLogger: true })
);

exports.run = run;
